@@ -17,10 +17,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
+#include <config.h>
+
 #include "dbus-signature.h"
 #include "dbus-marshal-recursive.h"
 #include "dbus-marshal-basic.h"
@@ -355,6 +357,7 @@ dbus_type_is_fixed (int typecode)
     case DBUS_TYPE_INT64:
     case DBUS_TYPE_UINT64:
     case DBUS_TYPE_DOUBLE:
+    case DBUS_TYPE_UNIX_FD:
       return TRUE;
     default:
       return FALSE;
