@@ -1,11 +1,11 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /* dbus-sysdeps-unix.h UNIX-specific wrappers around system/libc features (internal to D-Bus implementation)
- * 
+ *
  * Copyright (C) 2002, 2003, 2006  Red Hat, Inc.
  * Copyright (C) 2003 CodeFactory AB
  *
  * Licensed under the Academic Free License version 2.1
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -15,7 +15,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -25,7 +25,6 @@
 #ifndef DBUS_SYSDEPS_UNIX_H
 #define DBUS_SYSDEPS_UNIX_H
 
-#include <config.h>
 #include <dbus/dbus-sysdeps.h>
 
 #ifdef DBUS_WIN
@@ -41,19 +40,21 @@ DBUS_BEGIN_DECLS
  * @{
  */
 
-dbus_bool_t 
+dbus_bool_t
 _dbus_close     (int               fd,
                  DBusError        *error);
-int 
+int _dbus_dup   (int               fd,
+                 DBusError        *error);
+int
 _dbus_read      (int               fd,
                  DBusString       *buffer,
                  int               count);
-int 
+int
 _dbus_write     (int               fd,
                  const DBusString *buffer,
                  int               start,
                  int               len);
-int 
+int
 _dbus_write_two (int               fd,
                  const DBusString *buffer1,
                  int               start1,
@@ -71,12 +72,19 @@ int _dbus_listen_unix_socket  (const char     *path,
                                dbus_bool_t     abstract,
                                DBusError      *error);
 
+int _dbus_listen_systemd_sockets (int       **fd,
+                                 DBusError *error);
+
 dbus_bool_t _dbus_read_credentials (int               client_fd,
                                     DBusCredentials  *credentials,
                                     DBusError        *error);
 dbus_bool_t _dbus_send_credentials (int              server_fd,
                                     DBusError       *error);
 
+dbus_bool_t _dbus_lookup_launchd_socket (DBusString *socket_path,
+                                         const char *launchd_env_var,
+                                         DBusError  *error);
+
 /** Information about a UNIX user */
 typedef struct DBusUserInfo  DBusUserInfo;
 /** Information about a UNIX group */
